@@ -188,13 +188,17 @@ class RandomForestWithInstances(BaseModel):
             data.add_data_point(row_X, row_y)
         return data
 
-    def _predict(self, X: np.ndarray) -> typing.Tuple[np.ndarray, np.ndarray]:
+    def _predict(self, X: np.ndarray,
+                 cov_return_type: typing.Optional[str] = 'diagonal_cov') \
+            -> typing.Tuple[np.ndarray, np.ndarray]:
         """Predict means and variances for given X.
 
         Parameters
         ----------
         X : np.ndarray of shape = [n_samples,
                                    n_features (config + instance features)]
+        cov_return_type: typing.Optional[str]
+            Specifies what to return along with the mean. Refer ``predict()`` for more information.
 
         Returns
         -------
@@ -208,6 +212,8 @@ class RandomForestWithInstances(BaseModel):
                 'Expected 2d array, got %dd array!' % len(X.shape))
         if X.shape[1] != len(self.types):
             raise ValueError('Rows in X should have %d entries but have %d!' % (len(self.types), X.shape[1]))
+        if cov_return_type != 'diagonal_cov':
+            raise ValueError("'cov_return_type' can only take 'diagonal_cov' for this model")
 
         X = self._impute_inactive(X)
 
@@ -276,6 +282,8 @@ class RandomForestWithInstances(BaseModel):
         if self.instance_features is None or \
                 len(self.instance_features) == 0:
             mean_, var = self.predict(X)
+            assert var is not None  # please mypy
+
             var[var < self.var_threshold] = self.var_threshold
             var[np.isnan(var)] = self.var_threshold
             return mean_, var
@@ -290,8 +298,7 @@ class RandomForestWithInstances(BaseModel):
 
         X = self._impute_inactive(X)
 
-        mean_ = np.zeros(X.shape[0])
-        var = np.zeros(X.shape[0])
+        dat_ = np.zeros((X.shape[0], self.rf_opts.num_trees))  # marginalized predictions for each tree
         for i, x in enumerate(X):
 
             # marginalize over instances
@@ -305,21 +312,19 @@ class RandomForestWithInstances(BaseModel):
                     preds_trees[tree_id] += preds
 
             # 2. average in each tree
-            for tree_id in range(self.rf_opts.num_trees):
-                if self.log_y:
-                    preds_trees[tree_id] = \
-                        np.log(np.mean(np.exp(preds_trees[tree_id])))
-                else:
-                    preds_trees[tree_id] = np.mean(preds_trees[tree_id])
+            if self.log_y:
+                for tree_id in range(self.rf_opts.num_trees):
+                    dat_[i, tree_id] = \
+                        np.log(np.exp(np.array(preds_trees[tree_id])).mean())
+            else:
+                for tree_id in range(self.rf_opts.num_trees):
+                    dat_[i, tree_id] = np.array(preds_trees[tree_id]).mean()
 
-            # 3. compute statistics across trees
-            mean_x = np.mean(preds_trees)
-            var_x = np.var(preds_trees)
-            if var_x < self.var_threshold:
-                var_x = self.var_threshold
+        # 3. compute statistics across trees
+        mean_ = dat_.mean(axis=1)
+        var = dat_.var(axis=1)
 
-            var[i] = var_x
-            mean_[i] = mean_x
+        var[var < self.var_threshold] = self.var_threshold
 
         if len(mean_.shape) == 1:
             mean_ = mean_.reshape((-1, 1))
