@@ -345,7 +345,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
                 inc, seed=self.rng.randint(low=0, high=100000)))
             local_search_steps[i] += 1
         # Keeping track of configurations with equal acquisition value for plateau walking
-        neighbors_w_equal_acq = [[]] * num_candidates  # type: List[List[Configuration]]
+        neighbors_w_equal_acq = [[] for _ in range(num_candidates)]  # type: List[List[Configuration]]
 
         num_iters = 0
         while np.any(active):
@@ -574,13 +574,13 @@ class RandomSearch(AcquisitionFunctionMaximizer):
             return [(0, rand_configs[i]) for i in range(len(rand_configs))]
 
 
-class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
+class LocalAndSortedRandomSearch(AcquisitionFunctionMaximizer):
     """Implements SMAC's default acquisition function optimization.
 
     This optimizer performs local search from the previous best points
     according, to the acquisition function, uses the acquisition function to
-    sort randomly sampled configurations and interleaves unsorted, randomly
-    sampled configurations in between.
+    sort randomly sampled configurations. Random configurations are
+    interleaved by the main SMAC code.
 
     Parameters
     ----------
@@ -751,4 +751,6 @@ class FixedSet(AcquisitionFunctionMaximizer):
         num_points: int,
     ) -> List[Tuple[float, Configuration]]:
         configurations = copy.deepcopy(self.configurations)
+        for config in configurations:
+            config.origin = 'Fixed Set'
         return self._sort_configs_by_acq_value(configurations)
