@@ -4,23 +4,31 @@ import numpy as np
 import time
 import typing
 
+from smac.callbacks import IncorporateRunResultCallback
 from smac.configspace import Configuration
 from smac.epm.rf_with_instances import RandomForestWithInstances
 from smac.initial_design.initial_design import InitialDesign
-from smac.intensification.abstract_racer import AbstractRacer
+from smac.intensification.abstract_racer import AbstractRacer, RunInfoIntent
 from smac.optimizer import pSMAC
 from smac.optimizer.acquisition import AbstractAcquisitionFunction
 from smac.optimizer.random_configuration_chooser import ChooserNoCoolDown, RandomConfigurationChooser
 from smac.optimizer.ei_optimization import AcquisitionFunctionMaximizer
 from smac.optimizer.epm_configuration_chooser import EPMChooser
-from smac.runhistory.runhistory import RunHistory
+from smac.runhistory.runhistory import RunHistory, RunInfo, RunValue
 from smac.runhistory.runhistory2epm import AbstractRunHistory2EPM
 from smac.scenario.scenario import Scenario
 from smac.stats.stats import Stats
-from smac.tae.execute_ta_run import FirstRunCrashedException
+from smac.utils.constants import MAXINT
+from smac.tae import (
+    FirstRunCrashedException,
+    StatusType,
+    TAEAbortException,
+)
+from smac.tae.base import BaseRunner
 from smac.utils.io.traj_logging import TrajLogger
 from smac.utils.validate import Validator
 
+
 __author__ = "Aaron Klein, Marius Lindauer, Matthias Feurer"
 __copyright__ = "Copyright 2015, ML4AAD"
 __license__ = "3-clause BSD"
@@ -43,6 +51,7 @@ class SMBO(object):
     rng
     initial_design_configs
     epm_chooser
+    tae_runner
     """
 
     def __init__(self,
@@ -57,6 +66,7 @@ class SMBO(object):
                  acq_optimizer: AcquisitionFunctionMaximizer,
                  acquisition_func: AbstractAcquisitionFunction,
                  rng: np.random.RandomState,
+                 tae_runner: BaseRunner,
                  restore_incumbent: Configuration = None,
                  random_configuration_chooser: typing.Union[RandomConfigurationChooser] = ChooserNoCoolDown(2.0),
                  predict_x_best: bool = True,
@@ -92,6 +102,8 @@ class SMBO(object):
             incumbent to be used from the start. ONLY used to restore states.
         rng: np.random.RandomState
             Random number generator
+        tae_runner : smac.tae.base.BaseRunner Object
+            target algorithm run executor
         random_configuration_chooser
             Chooser for random configuration -- one of
             * ChooserNoCoolDown(modulus)
@@ -114,6 +126,8 @@ class SMBO(object):
         self.intensifier = intensifier
         self.num_run = num_run
         self.rng = rng
+        self._min_time = 10 ** -5
+        self.tae_runner = tae_runner
 
         self.initial_design_configs = []  # type: typing.List[Configuration]
 
@@ -131,6 +145,18 @@ class SMBO(object):
                                       predict_x_best=predict_x_best,
                                       min_samples_model=min_samples_model)
 
+        # Internal variable - if this is set to True it will gracefully stop SMAC
+        self._stop = False
+
+        # Callbacks. All known callbacks have a key. If something does not have a key here, there is
+        # no callback available.
+        self._callbacks = {
+            '_incorporate_run_results': list()
+        }  # type: typing.Dict[str, typing.List[typing.Callable]]
+        self._callback_to_key = {
+            IncorporateRunResultCallback: '_incorporate_run_results',
+        }  # type: typing.Dict[typing.Type, str]
+
     def start(self) -> None:
         """Starts the Bayesian Optimization loop.
         Detects whether the optimization is restored from a previous state.
@@ -138,7 +164,7 @@ class SMBO(object):
         self.stats.start_timing()
 
         # Initialization, depends on input
-        if self.stats.ta_runs == 0 and self.incumbent is None:
+        if self.stats.submitted_ta_runs == 0 and self.incumbent is None:
             self.logger.info('Running initial design')
             # Intensifier initialization
             self.initial_design_configs = self.initial_design.select_configurations()
@@ -147,14 +173,14 @@ class SMBO(object):
             if not self.initial_design_configs:
                 self.initial_design_configs = [self.config_space.get_default_configuration()]
 
-        elif self.stats.ta_runs > 0 and self.incumbent is None:
-            raise ValueError("According to stats there have been runs performed, "
+        elif self.stats.submitted_ta_runs > 0 and self.incumbent is None:
+            raise ValueError("According to stats there have been runs started, "
                              "but the optimizer cannot detect an incumbent. Did "
                              "you set the incumbent (e.g. after restoring state)?")
-        elif self.stats.ta_runs == 0 and self.incumbent is not None:
+        elif self.stats.submitted_ta_runs == 0 and self.incumbent is not None:
             raise ValueError("An incumbent is specified, but there are no runs "
-                             "recorded in the Stats-object. If you're restoring "
-                             "a state, please provide the Stats-object.")
+                             "recorded as started in the Stats-object. If you're "
+                             "restoring a state, please provide the Stats-object.")
         else:
             # Restoring state!
             self.logger.info("State Restored! Starting optimization with "
@@ -184,15 +210,17 @@ class SMBO(object):
 
             # sample next configuration for intensification
             # Initial design runs are also included in the BO loop now.
-            challenger, new_challenger = self.intensifier.get_next_challenger(
+            intent, run_info = self.intensifier.get_next_run(
                 challengers=self.initial_design_configs,
+                incumbent=self.incumbent,
                 chooser=self.epm_chooser,
                 run_history=self.runhistory,
-                repeat_configs=self.intensifier.repeat_configs
+                repeat_configs=self.intensifier.repeat_configs,
+                num_workers=self.tae_runner.num_workers(),
             )
 
             # remove config from initial design challengers to not repeat it again
-            self.initial_design_configs = [c for c in self.initial_design_configs if c != challenger]
+            self.initial_design_configs = [c for c in self.initial_design_configs if c != run_info.config]
 
             # update timebound only if a 'new' configuration is sampled as the challenger
             if self.intensifier.num_run == 0:
@@ -205,30 +233,91 @@ class SMBO(object):
                 time_left = self._get_timebound_for_intensification(time_spent, update=True)
                 self.logger.debug('Updated intensification time bound from %f to %f', old_time_left, time_left)
 
-            if challenger:
+            # Skip starting new runs if the budget is now exhausted
+            if self.stats.is_budget_exhausted():
+                intent = RunInfoIntent.SKIP
 
-                try:
-                    self.incumbent, inc_perf = self.intensifier.eval_challenger(
-                        challenger=challenger,
-                        incumbent=self.incumbent,
-                        run_history=self.runhistory,
-                        time_bound=max(self.intensifier._min_time, time_left))
+            # Skip the run if there was a request to do so.
+            # For example, during intensifier intensification, we
+            # don't want to rerun a config that was previously ran
+            if intent == RunInfoIntent.RUN:
+                # Track the fact that a run was launched in the run
+                # history. It's status is tagged as RUNNING, and once
+                # completed and processed, it will be updated accordingly
+                self.runhistory.add(
+                    config=run_info.config,
+                    cost=float(MAXINT),
+                    time=0.0,
+                    status=StatusType.RUNNING,
+                    instance_id=run_info.instance,
+                    seed=run_info.seed,
+                    budget=run_info.budget,
+                )
 
-                except FirstRunCrashedException:
-                    if self.scenario.abort_on_first_run_crash:  # type: ignore[attr-defined] # noqa F821
-                        raise
-                if self.scenario.shared_model:  # type: ignore[attr-defined] # noqa F821
-                    assert self.scenario.output_dir_for_this_run is not None  # please mypy
-                    pSMAC.write(run_history=self.runhistory,
-                                output_directory=self.scenario.output_dir_for_this_run,  # type: ignore[attr-defined] # noqa F821
-                                logger=self.logger)
+                run_info.config.config_id = self.runhistory.config_ids[run_info.config]
+
+                self.tae_runner.submit_run(run_info=run_info)
+
+                # There are 2 criteria that the stats object uses to know
+                # if the budged was exhausted.
+                # The budget time, which can only be known when the run finishes,
+                # And the number of ta executions. Because we submit the job at this point,
+                # we count this submission as a run. This prevent for using more
+                # runner runs than what the scenario allows
+                self.stats.submitted_ta_runs += 1
+
+            elif intent == RunInfoIntent.SKIP:
+                # No launch is required
+                # This marks a transition request from the intensifier
+                # To a new iteration
+                pass
+            elif intent == RunInfoIntent.WAIT:
+                # In any other case, we wait for resources
+                # This likely indicates that no further decision
+                # can be taken by the intensifier until more data is
+                # available
+                self.tae_runner.wait()
+            else:
+                raise NotImplementedError("No other RunInfoIntent has been coded!")
+
+            # Check if there is any result, or else continue
+            for run_info, result in self.tae_runner.get_finished_runs():
+
+                # Add the results of the run to the run history
+                # Additionally check for new incumbent
+                self._incorporate_run_results(run_info, result, time_left)
+
+            if self.scenario.shared_model:  # type: ignore[attr-defined] # noqa F821
+                assert self.scenario.output_dir_for_this_run is not None  # please mypy
+                pSMAC.write(run_history=self.runhistory,
+                            output_directory=self.scenario.output_dir_for_this_run,  # type: ignore[attr-defined] # noqa F821
+                            logger=self.logger)
 
             self.logger.debug("Remaining budget: %f (wallclock), %f (ta costs), %f (target runs)" % (
                 self.stats.get_remaing_time_budget(),
                 self.stats.get_remaining_ta_budget(),
                 self.stats.get_remaining_ta_runs()))
 
-            if self.stats.is_budget_exhausted():
+            if self.stats.is_budget_exhausted() or self._stop:
+                if self.stats.is_budget_exhausted():
+                    self.logger.debug("Exhausted configuration budget")
+                else:
+                    self.logger.debug("Shutting down because a configuration returned status STOP")
+
+                # The budget can be exhausted  for 2 reasons: number of ta runs or
+                # time. If the number of ta runs is reached, but there is still budget,
+                # wait for the runs to finish
+                while self.tae_runner.pending_runs():
+
+                    self.tae_runner.wait()
+
+                    for run_info, result in self.tae_runner.get_finished_runs():
+                        # Add the results of the run to the run history
+                        # Additionally check for new incumbent
+                        self._incorporate_run_results(run_info, result, time_left)
+
+                # Break from the intensification loop,
+                # as there are no more resources
                 break
 
             # print stats at the end of each intensification iteration
@@ -293,7 +382,7 @@ class SMBO(object):
         else:
             new_rh = validator.validate(config_mode, instance_mode, repetitions,
                                         n_jobs, backend, self.runhistory,
-                                        self.intensifier.tae_runner,
+                                        self.tae_runner,
                                         output_fn=new_rh_path)
         return new_rh
 
@@ -326,3 +415,81 @@ class SMBO(object):
                           "intensification: %.4f (%.2f)" %
                           (total_time, time_spent, (1 - frac_intensify), time_left, frac_intensify))
         return time_left
+
+    def _incorporate_run_results(self, run_info: RunInfo, result: RunValue,
+                                 time_left: float) -> None:
+        """
+        The SMBO submits a config-run-request via a RunInfo object.
+        When that config run is completed, a RunValue, which contains
+        all the relevant information obtained after running a job, is returned.
+        This method incorporates the status of that run into
+        the stats/runhistory objects so that other consumers
+        can advance with their task.
+
+        Additionally, it checks for a new incumbent via the intensifier process results,
+        which also has the side effect of moving the intensifier to a new state
+
+        Parameters
+        ----------
+        run_info: RunInfo
+            Describes the run (config) from which to process the results
+        result: RunValue
+            Contains relevant information regarding the execution of a config
+        time_left: float
+            time in [sec] available to perform intensify
+        """
+
+        # update SMAC stats
+        self.stats.ta_time_used += float(result.time)
+        self.stats.finished_ta_runs += 1
+
+        self.logger.debug(
+            "Return: Status: %r, cost: %f, time: %f, additional: %s" % (
+                result.status, result.cost, result.time, str(result.additional_info)
+            )
+        )
+
+        if result.status == StatusType.ABORT:
+            raise TAEAbortException("Target algorithm status ABORT - SMAC will "
+                                    "exit. The last incumbent can be found "
+                                    "in the trajectory-file.")
+        elif result.status == StatusType.STOP:
+            self._stop = True
+            return
+
+        self.runhistory.add(
+            config=run_info.config,
+            cost=result.cost,
+            time=result.time,
+            status=result.status,
+            instance_id=run_info.instance,
+            seed=run_info.seed,
+            budget=run_info.budget,
+            starttime=result.starttime,
+            endtime=result.endtime,
+            force_update=True,
+            additional_info=result.additional_info,
+        )
+        self.stats.n_configs = len(self.runhistory.config_ids)
+
+        if self.scenario.abort_on_first_run_crash :  # type: ignore[attr-defined] # noqa F821
+            if self.stats.finished_ta_runs == 1 and result.status == StatusType.CRASHED:
+                raise FirstRunCrashedException(
+                    "First run crashed, abort. Please check your setup -- we assume that your default "
+                    "configuration does not crashes. (To deactivate this exception, use the SMAC scenario option "
+                    "'abort_on_first_run_crash'). Additional run info: %s" % result.additional_info
+                )
+
+        # Update the intensifier with the result of the runs
+        self.incumbent, inc_perf = self.intensifier.process_results(
+            run_info=run_info,
+            incumbent=self.incumbent,
+            run_history=self.runhistory,
+            time_bound=max(self._min_time, time_left),
+            result=result,
+        )
+
+        for callback in self._callbacks['_incorporate_run_results']:
+            callback(smbo=self, run_info=run_info, result=result, time_left=time_left)
+
+        return
